@@ -118,6 +118,12 @@ class Version(object):
 
         self.partial = partial
 
+        # Cached precedence keys
+        # _cmp_precedence_key is used for semver-precedence comparison
+        self._cmp_precedence_key = self._build_precedence_key(with_build=False)
+        # _sort_precedence_key is used for self.precedence_key, esp. for sorted(...)
+        self._sort_precedence_key = self._build_precedence_key(with_build=True)
+
     @classmethod
     def _coerce(cls, value, allow_none=False):
         if value is None and allow_none:
@@ -244,6 +250,14 @@ class Version(object):
             while version.count('.') < 2:
                 version += '.0'
 
+        # Strip leading zeros in components
+        # Version is of the form nn, nn.pp or nn.pp.qq
+        version = '.'.join(
+            # If the part was '0', we end up with an empty string.
+            part.lstrip('0') or '0'
+            for part in version.split('.')
+        )
+
         if match.end() == len(version_string):
             return Version(version, partial=partial)
 
@@ -282,7 +296,8 @@ class Version(object):
 
     @classmethod
     def parse(cls, version_string, partial=False, coerce=False):
-        """Parse a version string into a Version() object.
+        """Parse a version string into a tuple of components:
+           (major, minor, patch, prerelease, build).
 
         Args:
             version_string (str), the version string to parse
@@ -399,11 +414,15 @@ class Version(object):
         # at least a field being `None`.
         return hash((self.major, self.minor, self.patch, self.prerelease, self.build))
 
-    @property
-    def precedence_key(self):
+    def _build_precedence_key(self, with_build=False):
+        """Build a precedence key.
+
+        The "build" component should only be used when sorting an iterable
+        of versions.
+        """
         if self.prerelease:
             prerelease_key = tuple(
-                NumericIdentifier(part) if re.match(r'^[0-9]+$', part) else AlphaIdentifier(part)
+                NumericIdentifier(part) if part.isdigit() else AlphaIdentifier(part)
                 for part in self.prerelease
             )
         else:
@@ -411,13 +430,31 @@ class Version(object):
                 MaxIdentifier(),
             )
 
+        if not with_build:
+            return (
+                self.major,
+                self.minor,
+                self.patch,
+                prerelease_key,
+            )
+
+        build_key = tuple(
+            NumericIdentifier(part) if part.isdigit() else AlphaIdentifier(part)
+            for part in self.build or ()
+        )
+
         return (
             self.major,
             self.minor,
             self.patch,
             prerelease_key,
+            build_key,
         )
 
+    @property
+    def precedence_key(self):
+        return self._sort_precedence_key
+
     def __cmp__(self, other):
         if not isinstance(other, self.__class__):
             return NotImplemented
@@ -449,22 +486,22 @@ class Version(object):
     def __lt__(self, other):
         if not isinstance(other, self.__class__):
             return NotImplemented
-        return self.precedence_key < other.precedence_key
+        return self._cmp_precedence_key < other._cmp_precedence_key
 
     def __le__(self, other):
         if not isinstance(other, self.__class__):
             return NotImplemented
-        return self.precedence_key <= other.precedence_key
+        return self._cmp_precedence_key <= other._cmp_precedence_key
 
     def __gt__(self, other):
         if not isinstance(other, self.__class__):
             return NotImplemented
-        return self.precedence_key > other.precedence_key
+        return self._cmp_precedence_key > other._cmp_precedence_key
 
     def __ge__(self, other):
         if not isinstance(other, self.__class__):
             return NotImplemented
-        return self.precedence_key >= other.precedence_key
+        return self._cmp_precedence_key >= other._cmp_precedence_key
 
 
 class SpecItem(object):
@@ -1115,7 +1152,7 @@ class SimpleSpec(BaseSpec):
                 elif minor is None:
                     return Range(Range.OP_GTE, target) & Range(Range.OP_LT, target.next_major())
                 elif patch is None:
-                    return Range(Range.OP_GTE, target) & Range(Range.OP_LT, target.next_patch())
+                    return Range(Range.OP_GTE, target) & Range(Range.OP_LT, target.next_minor())
                 elif build == '':
                     return Range(Range.OP_EQ, target, build_policy=Range.BUILD_STRICT)
                 else:
